package main

import (
	"encoding/json"
	"github.com/stretchr/testify/assert"
	"loafle.com/overflow/crawler_go"
	"loafle.com/overflow/of_rpc_go"
	"loafle.com/overflow/of_rpc_go/client"
	"loafle.com/overflow/of_rpc_go/models/param"
	"loafle.com/overflow/redis_protocol_crawler_go"
	"testing"
	"time"
	"loafle.com/overflow/ftps_protocol_crawler_go"
	"loafle.com/overflow/imap_protocol_crawler_go"
)

type configcall func()

////////////////////////////////////// redis rpc test start -->
func TestRedis(t *testing.T) {
	go initTestServer(func() {
		c := of_rpc.GetCrawler(crawler.REDIS.String()).(*redis_protocol_crawler_go.RedisHeahthCrawler)
		m := make(map[string]interface{}, 0)
		m["ip"] = "192.168.1.104"
		m["port"] = "6379"
		m["portType"] = "tcp"
		m["ssl"] = false
		c.PutConfig("test_redis_sid", m)
	})
	time.Sleep(2 * time.Second)
	in := param.Input{
		Name: crawler.REDIS.String(),
		Id:   "test_redis_sid",
	}
	checkhealth(t, in)
}
////////////////////////////////////// redis rpc test end <--

////////////////////////////////////// imap rpc test start -->
func TestImap(t *testing.T) {
	go initTestServer(func() {
		c := of_rpc.GetCrawler(crawler.IMAP.String()).(*imap_protocol_crawler_go.IMAPHealthCrawler)
		m := make(map[string]interface{}, 0)
		m["ip"] = "192.168.1.215"
		m["port"] = "993"
		m["portType"] = "tcp"
		m["ssl"] = true
		c.PutConfig("test_imap_sid", m)
	})
	time.Sleep(2 * time.Second)
	in := param.Input{
		Name: crawler.IMAP.String(),
		Id:   "test_imap_sid",
	}
	checkhealth(t, in)
}
////////////////////////////////////// imap rpc test end <--

////////////////////////////////////// ftp rpc test start -->
func TestFTP(t *testing.T) {
	go initTestServer(func() {
		c := of_rpc.GetCrawler(crawler.FTPS.String()).(*ftps_protocol_crawler_go.FTPSHealthCrawler)
		m := make(map[string]interface{}, 0)
		m["ip"] = "192.168.1.103"
		m["port"] = "2121"
		m["portType"] = "tcp"
		m["ssl"] = false
		c.PutConfig("test_ftps_sid", m)
	})
	time.Sleep(2 * time.Second)
	in := param.Input{
		Name: crawler.FTPS.String(),
		Id:   "test_ftps_sid",
	}
	checkhealth(t, in)
}
////////////////////////////////////// ftp rpc test end <--



func initTestServer(config configcall) {
	initCrawlers()
	config()
	start()
}

func checkhealth(t *testing.T, in param.Input) {
	result, err := client.InvokeJSONRPCGet("50000", in)

	if err != nil {
		t.Fatal(err)
	}

	var check bool
	json.Unmarshal(result.Data, &check)
	assert.Equal(t, true, check)
}
